@@ -63,7 +63,7 @@ DO_COMMAND(do_map)
 	int cnt;
 	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE];
 
-	arg = get_arg_in_braces(arg, arg1, FALSE);
+	arg = get_arg_in_braces(ses, arg, arg1, FALSE);
 
 	if (*arg1 == 0)
 	{
@@ -74,7 +74,7 @@ DO_COMMAND(do_map)
 		tintin_printf2(ses, "#map create   [size]                   (creates the initial map)");
 		tintin_printf2(ses, "#map destroy                           (destroys the map)");
 		tintin_printf2(ses, "#map delete   <direction>              (delete the room at given dir)");
-		tintin_printf2(ses, "#map dig      <direction> [new]        (creates a new room)");
+		tintin_printf2(ses, "#map dig      <direction> [new] [vnum] (creates a new room)");
 		tintin_printf2(ses, "#map exit     <direction>  <command>   (sets the exit command)");
 		tintin_printf2(ses, "#map exitflag <direction> <exit flag>  (set the exit direction)");
 		tintin_printf2(ses, "#map explore  <direction>              (saves path to #path)");
@@ -92,6 +92,7 @@ DO_COMMAND(do_map)
 		tintin_printf2(ses, "#map map      <radius>                 (shows an ascii map)");
 		tintin_printf2(ses, "#map name     <room name>              (set the room's name)");
 		tintin_printf2(ses, "#map read     <filename>               (load your map from a file)");
+		tintin_printf2(ses, "#map resize   <size>                   (resize the maximum size)");
 		tintin_printf2(ses, "#map roomflag <room flag>              (set room based flags)");
 		tintin_printf2(ses, "#map set      <option>     <value>     (set various values)");
 		tintin_printf2(ses, "#map return                            (return to last room.)");
@@ -269,6 +270,13 @@ DO_MAP(map_dig)
 	arg = sub_arg_in_braces(ses, arg, arg1, GET_ALL, SUB_VAR|SUB_FUN);
 	arg = sub_arg_in_braces(ses, arg, arg2, GET_ALL, SUB_VAR|SUB_FUN);
 
+	if (*arg1 == 0)
+	{
+		tintin_printf2(ses, "#SYNTAX: #MAP DIG {<DIRECTION>} {NEW|<VNUM>}");
+
+		return;
+	}
+
 	for (room = 1 ; room < ses->map->size ; room++)
 	{
 		if (ses->map->room_list[room] == NULL)
@@ -315,7 +323,6 @@ DO_MAP(map_dig)
 	if (strcasecmp(arg2, "new") && find_coord(ses, arg1))
 	{
 		room = map_search_tar_room;
-
 		show_message(ses, -1, "#MAP CREATE LINK %5d {%s}.", room, ses->map->room_list[room]->name);
 	}
 	else
@@ -323,6 +330,7 @@ DO_MAP(map_dig)
 		sprintf(temp, "{%d} {0} {} {%d} { }", room, room);
 		create_room(ses, temp);
 	}
+
 	sprintf(temp, "{%d} {%s} {%s}", room, arg1, arg1);
 	create_exit(ses, ses->map->in_room, temp);
 
@@ -338,7 +346,7 @@ DO_MAP(map_dig)
 
 DO_MAP(map_exit)
 {
-	char temp[BUFFER_SIZE], arg3[BUFFER_SIZE];
+	char arg3[BUFFER_SIZE];
 	struct exit_data *exit;
 	struct listnode *node;
 	int room;
@@ -355,16 +363,19 @@ DO_MAP(map_exit)
 
 		return;
 	}
-
-	if (is_abbrev(arg2, "COMMAND"))
+	if (*arg2 == 0)
 	{
-		sprintf(temp, "{%d} {%s} {%s}", exit->vnum, exit->name, arg3);
+		tintin_printf2(ses, "  command: %s", exit->cmd);
+		tintin_printf2(ses, "direction: %d", exit->dir);
+		tintin_printf2(ses, "    flags: %d", exit->flags);
+		tintin_printf2(ses, "     name: %s", exit->name);
+		tintin_printf2(ses, "     vnum: %d", exit->vnum);
+	}
+	else if (is_abbrev(arg2, "COMMAND"))
+	{
+		exit->cmd = restring(exit->cmd, arg3);
 
-		delete_exit(ses, ses->map->in_room, exit);
-
-		create_exit(ses, ses->map->in_room, temp);
-
-		show_message(ses, -1, "#MAP EXIT {%s} : COMMAND SET TO {%s}.", arg1, arg3);
+		show_message(ses, -1, "#MAP EXIT {%s} : COMMAND SET TO {%s}.", arg1, exit->cmd);
 	}
 	else if (is_abbrev(arg2, "DIRECTION"))
 	{
@@ -379,19 +390,34 @@ DO_MAP(map_exit)
 
 		show_message(ses, -1, "#MAP EXIT {%s} : DIRECTION SET TO {%s}.", arg1, arg3);
 	}
-	else if (is_abbrev(arg2, "FLAG"))
+	else if (is_abbrev(arg2, "FLAGS"))
 	{
-		show_message(ses, -1, "#MAP EXIT {%s} : FLAG SET TO {%s}.", arg1, arg3);		
+		exit->flags = atoi(arg3);
+
+		show_message(ses, -1, "#MAP EXIT {%s} : FLAGS SET TO {%d}.", arg1, exit->flags);
+	}
+	else if (is_abbrev(arg2, "GET"))
+	{
+		if (*arg3)
+		{
+			set_nest_node(ses->list[LIST_VARIABLE], arg3, "%s", exit->data);
+		}
+		else
+		{
+			tintin_printf2(ses, "#MAP EXIT GET: No destination variable.");
+		}
 	}
 	else if (is_abbrev(arg2, "NAME"))
 	{
-		sprintf(temp, "{%d} {%s} {%s}", exit->vnum, arg3, arg3);
+		exit->name = restring(exit->name, arg3);
 
-		delete_exit(ses, ses->map->in_room, exit);
+		show_message(ses, -1, "#MAP EXIT {%s} : NAME SET TO {%s}.", arg1, exit->name);
+	}
+	else if (is_abbrev(arg2, "SET"))
+	{
+		exit->data = restring(exit->data, arg3);
 
-		create_exit(ses, ses->map->in_room, temp);
-
-		show_message(ses, -1, "#MAP EXIT {%s} : NAME SET TO {%s}.", arg1, arg3);
+		show_message(ses, -1, "#MAP EXIT {%s} : DATA SET TO {%s}.", arg1, exit->data);
 	}
 	else if (is_abbrev(arg2, "VNUM"))
 	{
@@ -414,7 +440,7 @@ DO_MAP(map_exit)
 	}
 	else
 	{
-		tintin_printf2(ses, "Syntax: #MAP EXIT {<NAME>} {COMMAND|DIRECTION|NAME|FLAG} {<argument>}");
+		tintin_printf2(ses, "Syntax: #MAP EXIT {<NAME>} {COMMAND|DIRECTION|NAME|FLAGS|VNUM} {<argument>}");
 	}
 }
 
@@ -450,6 +476,12 @@ DO_MAP(map_exitflag)
 			return;
 		}
 	}
+	else
+	{
+		tintin_printf2(ses, "#MAP: Hide flag is set to %s.", HAS_BIT(exit->flags, EXIT_FLAG_HIDE) ? "on" : "off");
+
+		return;
+	}
 
 	if (is_abbrev("on", arg3))
 	{
@@ -622,14 +654,19 @@ DO_MAP(map_get)
 	}
 	else if (*arg1 == 0 || *arg2 == 0)
 	{
-		tintin_printf2(ses, "  roomvnum: %d", room->vnum);
-		tintin_printf2(ses, "  roomarea: %s", room->area);
-		tintin_printf2(ses, " roomcolor: %s", room->color);
-		tintin_printf2(ses, "  roomdesc: %s", room->desc);
-		tintin_printf2(ses, " roomflags: %d", room->flags);
-		tintin_printf2(ses, "  roomname: %s", room->name);
-		tintin_printf2(ses, "  roomnote: %s", room->note);
-		tintin_printf2(ses, "roomsymbol: %s", room->symbol);
+		tintin_printf2(ses, "   roomvnum: %d", room->vnum);
+		tintin_printf2(ses, "   roomarea: %s", room->area);
+		tintin_printf2(ses, "  roomcolor: %s", room->color);
+		tintin_printf2(ses, "   roomdata: %s", room->data);
+		tintin_printf2(ses, "   roomdesc: %s", room->desc);
+		tintin_printf2(ses, "  roomexits: %d", get_map_exits(ses, room->vnum));
+		tintin_printf2(ses, "  roomflags: %d", room->flags);
+		tintin_printf2(ses, "   roomname: %s", room->name);
+		tintin_printf2(ses, "   roomnote: %s", room->note);
+		tintin_printf2(ses, " roomsymbol: %s", room->symbol);
+		tintin_printf2(ses, "roomterrain: %s", room->terrain);
+		tintin_printf2(ses, " worldflags: %d", ses->map->flags);
+		tintin_printf2(ses, "  worldsize: %d", ses->map->size);
 	}
 	else
 	{
@@ -641,6 +678,10 @@ DO_MAP(map_get)
 		{
 			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%s", room->color);
 		}
+		else if (is_abbrev(arg1, "roomdata"))
+		{
+			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%s", room->data);
+		}
 		else if (is_abbrev(arg1, "roomdesc"))
 		{
 			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%s", room->desc);
@@ -661,6 +702,10 @@ DO_MAP(map_get)
 		{
 			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%s", room->symbol);
 		}
+		else if (is_abbrev(arg1, "roomterrain"))
+		{
+			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%s", room->terrain);
+		}
 		else if (is_abbrev(arg1, "roomvnum"))
 		{
 			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%d", room->vnum);
@@ -675,6 +720,14 @@ DO_MAP(map_get)
 			}
 			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%s", buf);
 		}
+		else if (is_abbrev(arg1, "worldflags"))
+		{
+			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%d", ses->map->flags);
+		}
+		else if (is_abbrev(arg1, "worldsize"))
+		{
+			set_nest_node(ses->list[LIST_VARIABLE], arg2, "%d", ses->map->size);
+		}
 		else
 		{
 			show_message(ses, -1, "#MAP GET: unknown option: %s.", arg1);
@@ -716,25 +769,38 @@ DO_MAP(map_info)
 		}
 	}
 
-	tintin_printf2(ses, "%-20s %d/%d", "Total rooms:",    cnt, ses->map->size);
-	tintin_printf2(ses, "%-20s %d", "Total exits:",    exits);
-	tintin_printf2(ses, "%-20s %s", "Vtmap:",          HAS_BIT(ses->map->flags, MAP_FLAG_VTMAP) ? "on" : "off");
-	tintin_printf2(ses, "%-20s %s", "Static:",         HAS_BIT(ses->map->flags, MAP_FLAG_STATIC) ? "on" : "off");
+	tintin_printf2(ses, "%-20s%-20d%-20s%-20d", "Total rooms:", cnt, "Total exits:", exits);
+	tintin_printf2(ses, "%-20s%-20d%-20s%-20d", "World size:", ses->map->size, "Current room:", ses->map->in_room);
+
+	tintin_printf2(ses, "");
+
+	tintin_printf2(ses, "%-14s%5s %-14s%5s %-14s%5s %-14s%5s", "Vtmap:", HAS_BIT(ses->map->flags, MAP_FLAG_VTMAP) ? "on" : "off", "Static:", HAS_BIT(ses->map->flags, MAP_FLAG_STATIC) ? "on" : "off", "Vtgraphics", HAS_BIT(ses->map->flags, MAP_FLAG_VTGRAPHICS) ? "on" : "off", "Asciigraphics", HAS_BIT(ses->map->flags, MAP_FLAG_ASCIIGRAPHICS) ? "on" : "off");
+	tintin_printf2(ses, "%-14s%5s %-14s%5s", "Asciivnums:", HAS_BIT(ses->map->flags, MAP_FLAG_ASCIIVNUMS) ? "on" : "off", "Nofollow:", HAS_BIT(ses->map->flags, MAP_FLAG_NOFOLLOW) ? "on" : "off");
 
 	if (ses->map->in_room)
 	{
 		tintin_printf2(ses, "");
-		tintin_printf2(ses, "%-20s %d", "Room vnum:",      ses->map->in_room);
-		tintin_printf2(ses, "%-20s %s", "Room name:",      ses->map->room_list[ses->map->in_room]->name);
-		tintin_printf2(ses, "%-20s %d", "Room size:",      ses->map->room_list[ses->map->in_room]->size);
-		tintin_printf2(ses, "%-20s %s", "Avoid:",          HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_AVOID) ? "on" : "off");
-		tintin_printf2(ses, "%-20s %s", "Hide:",           HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_HIDE) ? "on" : "off");		
-		tintin_printf2(ses, "%-20s %s", "Leave:",          HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_LEAVE) ? "on" : "off");
-		tintin_printf2(ses, "%-20s %s", "Void:",           HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_VOID) ? "on" : "off");
+
+		tintin_printf2(ses, "%-20s%s", "Room area:",    ses->map->room_list[ses->map->in_room]->area);
+		tintin_printf2(ses, "%-20s%s", "Room data:",    ses->map->room_list[ses->map->in_room]->data);
+		tintin_printf2(ses, "%-20s%s", "Room desc:",    ses->map->room_list[ses->map->in_room]->desc);
+		tintin_printf2(ses, "%-20s%s", "Room name:",    ses->map->room_list[ses->map->in_room]->name);
+		tintin_printf2(ses, "%-20s%s", "Room note:",    ses->map->room_list[ses->map->in_room]->note);
+		tintin_printf2(ses, "%-20s%s", "Room symbol:",  ses->map->room_list[ses->map->in_room]->symbol);
+		tintin_printf2(ses, "%-20s%s", "Room terrain:", ses->map->room_list[ses->map->in_room]->terrain);
+		tintin_printf2(ses, "%-20s%d", "Room vnum:",    ses->map->in_room);
+
+		tintin_printf2(ses, "");
+
+		tintin_printf2(ses, "%-14s%5s %-14s%5s %-14s%5s %-14s%5s", "Avoid:", HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_AVOID) ? "on" : "off", "Hide:", HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_HIDE) ? "on" : "off", "Leave:",  HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_LEAVE) ? "on" : "off", "Static:", HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_STATIC) ? "on" : "off");
+		tintin_printf2(ses, "%-14s%5s",                             "Void:", HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_VOID) ? "on" : "off");
+
+		tintin_printf2(ses, "");
 
 		for (exit = ses->map->room_list[ses->map->in_room]->f_exit ; exit ; exit = exit->next)
 		{
-			tintin_printf2(ses, "%-20s %03d %-3s (%s) to room: %-5d (%s)", "Exit:", exit->dir, exit->name, exit->cmd, exit->vnum, ses->map->room_list[exit->vnum]->name);
+			tintin_printf2(ses, "%-14s%5s %-14s%5d %-14s%5d %-14s%5s", "Exit name:", exit->name, "Exit vnum:", exit->vnum, "Exit flags:", exit->flags, "Exit command:", exit->cmd);
+			tintin_printf2(ses, "%-14s%s", "Exit data:", exit->data);
 		}
 	}
 }
@@ -813,10 +879,12 @@ DO_MAP(map_insert)
 
 DO_MAP(map_jump)
 {
+	char arg3[BUFFER_SIZE];
 	int vnum;
 
 	arg = sub_arg_in_braces(ses, arg, arg1, GET_ONE, SUB_VAR|SUB_FUN);
-	arg = sub_arg_in_braces(ses, arg, arg2, GET_ALL, SUB_VAR|SUB_FUN);
+	arg = sub_arg_in_braces(ses, arg, arg2, GET_ONE, SUB_VAR|SUB_FUN);
+	arg = sub_arg_in_braces(ses, arg, arg3, GET_ALL, SUB_VAR|SUB_FUN);
 
 	map_grid_x = 0;
 	map_grid_y = 0;
@@ -825,8 +893,9 @@ DO_MAP(map_jump)
 	map_search_ses       = ses;
 	map_search_tar_room  = 0;
 
-	map_grid_x += atoi(arg1);
-	map_grid_y += atoi(arg2);
+	map_grid_x += get_number(ses, arg1);
+	map_grid_y += get_number(ses, arg2);
+	map_grid_z += get_number(ses, arg3);
 
 	for (vnum = 0 ; vnum < ses->map->size ; vnum++)
 	{
@@ -848,7 +917,7 @@ DO_MAP(map_jump)
 	}
 	else
 	{
-		show_message(ses, -1, "#MAP: Couldn't find room at {%s} {%s}.", arg1, arg2);
+		show_message(ses, -1, "#MAP: Couldn't find room at {%d} {%d} {%d}.", map_grid_x, map_grid_y, map_grid_z);
 	}
 }
 
@@ -865,7 +934,7 @@ DO_MAP(map_leave)
 
 		show_message(ses, -1, "#MAP: Leaving the map. Use goto or return to return.");
 
-		check_all_events(ses, 0, 1, "MAP EXIT MAP", ntos(ses->map->in_room));
+		check_all_events(ses, SUB_ARG|SUB_SEC, 0, 1, "MAP EXIT MAP", ntos(ses->map->in_room));
 	}
 }
 
@@ -1252,6 +1321,27 @@ DO_MAP(map_read)
 
 }
 
+DO_MAP(map_resize)
+{
+	int size;
+
+	arg = sub_arg_in_braces(ses, arg, arg1, GET_ALL, SUB_VAR|SUB_FUN);
+
+	size = atoi(arg1);
+
+	if (size <= ses->map->size)
+	{
+		tintin_printf2(ses, "#MAP RESIZE: NEW SIZE (%d) MUST BE GREATER THAN CURRENT SIZE (%d).", size, ses->map->size);
+		return;
+	}
+
+	ses->map->size = size;
+
+	ses->map->room_list = (struct room_data **) realloc(ses->map->room_list, ses->map->size * sizeof(struct room_data *));
+
+	show_message(ses, -1, "#MAP RESIZE: MAP RESIZED TO %d ROOMS.", size);
+}
+
 DO_MAP(map_return)
 {
 	if (ses->map == NULL || ses->map->room_list[ses->map->last_room] == NULL)
@@ -1391,6 +1481,7 @@ DO_MAP(map_set)
 	{
 		tintin_printf2(ses, "  roomarea: %s", room->area);
 		tintin_printf2(ses, " roomcolor: %s", room->color);
+		tintin_printf2(ses, "  roomdata: %s", room->data);
 		tintin_printf2(ses, "  roomdesc: %s", room->desc);
 		tintin_printf2(ses, " roomflags: %d", room->flags);
 		tintin_printf2(ses, "  roomname: %s", room->name);
@@ -1409,6 +1500,11 @@ DO_MAP(map_set)
 			RESTRING(room->color, arg2);
 			show_message(ses, -1, "#MAP SET: roomcolor set to: %s", arg2);
 		}
+		else if (is_abbrev(arg1, "roomdata"))
+		{
+			RESTRING(room->data, arg2);
+			show_message(ses, -1, "#MAP SET: roomdata set to: %s", arg2);
+		}
 		else if (is_abbrev(arg1, "roomdesc"))
 		{
 			RESTRING(room->desc, arg2);
@@ -1437,6 +1533,11 @@ DO_MAP(map_set)
 
 			show_message(ses, -1, "#MAP SET: roomsymbol set to: %s", room->symbol);
 		}
+		else if (is_abbrev(arg1, "roomterrain"))
+		{
+			RESTRING(room->terrain, arg2);
+			show_message(ses, -1, "#MAP SET: roomterrain set to: %s", arg2);
+		}
 		else
 		{
 			show_message(ses, -1, "#MAP SET: unknown option: %s", arg1);
@@ -1643,7 +1744,7 @@ DO_MAP(map_write)
 	{
 		if (ses->map->room_list[vnum])
 		{
-			fprintf(file, "\nR {%5d} {%d} {%s} {%s} {%s} {%s} {%s} {%s}\n",
+			fprintf(file, "\nR {%5d} {%d} {%s} {%s} {%s} {%s} {%s} {%s} {%s} {%s}\n",
 				ses->map->room_list[vnum]->vnum,
 				ses->map->room_list[vnum]->flags,
 				ses->map->room_list[vnum]->color,
@@ -1651,16 +1752,19 @@ DO_MAP(map_write)
 				ses->map->room_list[vnum]->symbol,
 				ses->map->room_list[vnum]->desc,
 				ses->map->room_list[vnum]->area,
-				ses->map->room_list[vnum]->note);
-                        
+				ses->map->room_list[vnum]->note,
+				ses->map->room_list[vnum]->terrain,
+				ses->map->room_list[vnum]->data);
+
 			for (exit = ses->map->room_list[vnum]->f_exit ; exit ; exit = exit->next)
 			{
-				fprintf(file, "E {%5d} {%s} {%s} {%d} {%d}\n",
+				fprintf(file, "E {%5d} {%s} {%s} {%d} {%d} {%s}\n",
 					exit->vnum,
 					exit->name,
 					exit->cmd,
 					exit->dir,
-					exit->flags);
+					exit->flags,
+					exit->data);
 			}
 		}
 	}
@@ -1680,7 +1784,7 @@ void create_map(struct session *ses, char *arg)
 	}
 
 	ses->map = (struct map_data *) calloc(1, sizeof(struct map_data));
-	ses->map->size = atoi(arg) > 0 ? atoi(arg) : 15000;
+	ses->map->size = atoi(arg) > 0 ? atoi(arg) : 50000;
 
 	ses->map->room_list = (struct room_data **) calloc(ses->map->size, sizeof(struct room_data *));
 
@@ -1733,7 +1837,7 @@ void create_legend(struct session *ses, char *arg)
 
 	for (cnt = 0 ; cnt < 17 ; cnt++)
 	{
-		arg = get_arg_in_braces(arg, buf, FALSE);
+		arg = get_arg_in_braces(ses, arg, buf, FALSE);
 
 		substitute(ses, buf, buf, SUB_ESC);
 
@@ -1751,37 +1855,25 @@ void create_legend(struct session *ses, char *arg)
 
 int create_room(struct session *ses, char *arg)
 {
-	char vnum[BUFFER_SIZE], flags[BUFFER_SIZE], color[BUFFER_SIZE], name[BUFFER_SIZE], symbol[BUFFER_SIZE], desc[BUFFER_SIZE], area[BUFFER_SIZE], note[BUFFER_SIZE];
+	char buf[BUFFER_SIZE];
 	struct room_data *newroom;
 
-	arg = get_arg_in_braces(arg, vnum,   FALSE);
-	arg = get_arg_in_braces(arg, flags,  FALSE);
-	arg = get_arg_in_braces(arg, color,  FALSE);
-	arg = get_arg_in_braces(arg, name,   FALSE);
-	arg = get_arg_in_braces(arg, symbol, FALSE);
-	arg = get_arg_in_braces(arg, desc,   FALSE);
-	arg = get_arg_in_braces(arg, area,   FALSE);
-	arg = get_arg_in_braces(arg, note,   FALSE);
+	newroom          = (struct room_data *) calloc(1, sizeof(struct room_data));
 
-	/* Backward compatbility 1.96.5 */
-
-	if (*symbol == 0) { strcpy(symbol, " "); }
-
-	newroom         = (struct room_data *) calloc(1, sizeof(struct room_data));
-
-	newroom->color  = strdup(color);
-	newroom->name   = strdup(name);
-	newroom->symbol = strdup(symbol);
-	newroom->desc   = strdup(desc);
-	newroom->area   = strdup(area);
-	newroom->note   = strdup(note);
-
-	newroom->flags  = atoi(flags);
-	newroom->vnum   = atoi(vnum);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->vnum    = atoi(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->flags   = atoi(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->color   = strdup(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->name    = strdup(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->symbol  = strdup(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->desc    = strdup(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->area    = strdup(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->note    = strdup(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->terrain = strdup(buf);
+	arg = get_arg_in_braces(ses, arg, buf, FALSE); newroom->data    = strdup(buf);
 
 	ses->map->room_list[newroom->vnum] = newroom;
 
-	show_message(ses, -1, "#MAP CREATE ROOM %5s {%s}.", vnum, name);
+	show_message(ses, -1, "#MAP CREATE ROOM %5d {%s}.", newroom->vnum, newroom->name);
 
 	return newroom->vnum;
 }
@@ -1832,20 +1924,23 @@ void create_exit(struct session *ses, int room, char *arg)
 	struct exit_data *newexit;
 	struct listnode *node;
 
-	char vnum[BUFFER_SIZE], name[BUFFER_SIZE], cmd[BUFFER_SIZE], dir[BUFFER_SIZE], flags[BUFFER_SIZE];
+	char vnum[BUFFER_SIZE], name[BUFFER_SIZE], cmd[BUFFER_SIZE], dir[BUFFER_SIZE], flags[BUFFER_SIZE], data[BUFFER_SIZE];
 
-	arg = get_arg_in_braces(arg, vnum,  FALSE);
-	arg = get_arg_in_braces(arg, name,  FALSE);
-	arg = get_arg_in_braces(arg, cmd,   TRUE);
-	arg = get_arg_in_braces(arg, dir,   FALSE);
-	arg = get_arg_in_braces(arg, flags, FALSE);
+	arg = get_arg_in_braces(ses, arg, vnum,  FALSE);
+	arg = get_arg_in_braces(ses, arg, name,  FALSE);
+	arg = get_arg_in_braces(ses, arg, cmd,   TRUE);
+	arg = get_arg_in_braces(ses, arg, dir,   FALSE);
+	arg = get_arg_in_braces(ses, arg, flags, FALSE);
+	arg = get_arg_in_braces(ses, arg, data,  TRUE);
 
 	newexit        = (struct exit_data *) calloc(1, sizeof(struct exit_data));
+
 	newexit->vnum  = atoi(vnum);
 	newexit->name  = strdup(name);
 	newexit->cmd   = strdup(cmd);
 	newexit->dir   = atoi(dir);
 	newexit->flags = atoi(flags);
+	newexit->data  = strdup(data);
 
 	if (newexit->dir == 0)
 	{
@@ -1890,14 +1985,14 @@ int get_map_exits(struct session *ses, int room)
 {
 	struct exit_data *exit;
 
-	int exits = 0;
+	int count = 0;
 
 	for (exit = ses->map->room_list[room]->f_exit ; exit ; exit = exit->next)
 	{
-		exits++;
+		count++;
 	}
 
-	return exits;
+	return count;
 }
 
 int follow_map(struct session *ses, char *argument)
@@ -2051,9 +2146,9 @@ void add_undo(struct session *ses, char *format, ...)
 	vsprintf(buf, format, args);
 	va_end(args);
 
-	arg = get_arg_in_braces(buf, dir, FALSE);
-	arg = get_arg_in_braces(arg, rev, FALSE);
-	arg = get_arg_in_braces(arg, flag, FALSE);
+	arg = get_arg_in_braces(ses, buf, dir, FALSE);
+	arg = get_arg_in_braces(ses, arg, rev, FALSE);
+	arg = get_arg_in_braces(ses, arg, flag, FALSE);
 
 	link = (struct link_data *) calloc(1, sizeof(struct link_data));
 
@@ -2418,7 +2513,7 @@ char *draw_room(struct session *ses, struct room_data *room, int line)
 						}
 						else if (strip_color_strlen(room->symbol) <= 1)
 						{
-							cat_sprintf(buf, "%s[%s%s]%s", HAS_BIT(room->flags, ROOM_FLAG_PATH) ? ses->map->path_color : *room->color ? room->color : ses->map->room_color, room->symbol, HAS_BIT(room->flags, ROOM_FLAG_PATH) ? ses->map->path_color : *room->color ? room->color : ses->map->room_color, ses->map->exit_color);
+							cat_sprintf(buf, "%s[%-1s%s]%s", HAS_BIT(room->flags, ROOM_FLAG_PATH) ? ses->map->path_color : *room->color ? room->color : ses->map->room_color, room->symbol, HAS_BIT(room->flags, ROOM_FLAG_PATH) ? ses->map->path_color : *room->color ? room->color : ses->map->room_color, ses->map->exit_color);
 						}
 						else
 						{
@@ -2617,7 +2712,7 @@ int match_room(struct session *ses, int room, char *arg1, char *arg2, char *arg3
 
 	if (*arg1)
 	{
-		if (!match(ses, ses->map->room_list[room]->name, arg1))
+		if (!match(ses, ses->map->room_list[room]->name, arg1, SUB_VAR|SUB_FUN))
 		{
 			return 0;
 		}
@@ -2631,7 +2726,7 @@ int match_room(struct session *ses, int room, char *arg1, char *arg2, char *arg3
 
 		while (*arg)
 		{
-			arg = get_arg_in_braces(arg, exit, GET_ONE);
+			arg = get_arg_in_braces(ses, arg, exit, GET_ONE);
 
 			size++;
 
@@ -2655,7 +2750,7 @@ int match_room(struct session *ses, int room, char *arg1, char *arg2, char *arg3
 
 	if (*arg3)
 	{
-		if (!match(ses, ses->map->room_list[room]->desc, arg3))
+		if (!match(ses, ses->map->room_list[room]->desc, arg3, SUB_VAR|SUB_FUN))
 		{
 			return 0;
 		}
@@ -2663,7 +2758,7 @@ int match_room(struct session *ses, int room, char *arg1, char *arg2, char *arg3
 
 	if (*arg4)
 	{
-		if (!match(ses, ses->map->room_list[room]->area, arg4))
+		if (!match(ses, ses->map->room_list[room]->area, arg4, SUB_VAR|SUB_FUN))
 		{
 			return 0;
 		}
@@ -2671,7 +2766,7 @@ int match_room(struct session *ses, int room, char *arg1, char *arg2, char *arg3
 
 	if (*arg5)
 	{
-		if (!match(ses, ses->map->room_list[room]->note, arg5))
+		if (!match(ses, ses->map->room_list[room]->note, arg5, SUB_VAR|SUB_FUN))
 		{
 			return 0;
 		}
@@ -2745,8 +2840,8 @@ void goto_room(struct session *ses, int room)
 
 	if (ses->map->in_room)
 	{
-		check_all_events(ses, 0, 2, "MAP EXIT ROOM", ntos(last_room), ntos(room));
-		check_all_events(ses, 1, 2, "MAP EXIT ROOM %d", last_room, ntos(last_room), ntos(room));
+		check_all_events(ses, SUB_ARG|SUB_SEC, 0, 2, "MAP EXIT ROOM", ntos(last_room), ntos(room));
+		check_all_events(ses, SUB_ARG|SUB_SEC, 1, 2, "MAP EXIT ROOM %d", last_room, ntos(last_room), ntos(room));
 	}
 
 	ses->map->in_room = room;
@@ -2755,11 +2850,11 @@ void goto_room(struct session *ses, int room)
 
 	if (last_room == 0)
 	{
-		check_all_events(ses, 0, 1, "MAP ENTER MAP", ntos(room));
+		check_all_events(ses, SUB_ARG|SUB_SEC, 0, 1, "MAP ENTER MAP", ntos(room));
 	}
 
-	check_all_events(ses, 0, 2, "MAP ENTER ROOM", ntos(room), ntos(last_room));
-	check_all_events(ses, 1, 2, "MAP ENTER ROOM %d", room, ntos(room), ntos(last_room));
+	check_all_events(ses, SUB_ARG|SUB_SEC, 0, 2, "MAP ENTER ROOM", ntos(room), ntos(last_room));
+	check_all_events(ses, SUB_ARG|SUB_SEC, 1, 2, "MAP ENTER ROOM %d", room, ntos(room), ntos(last_room));
 
 	pop_call();
 	return;
