@@ -61,8 +61,9 @@ DO_COMMAND(do_math)
 
 	root = ses->list[LIST_VARIABLE];
 
-	arg = get_arg_in_braces(arg, left,  GET_NST);
-	arg = get_arg_in_braces(arg, right, GET_ALL);
+	arg = sub_arg_in_braces(ses, arg, left, GET_NST, SUB_VAR|SUB_FUN);
+
+	arg = get_arg_in_braces(ses, arg, right, GET_ALL);
 
 	if (*left == 0 || *right == 0)
 	{
@@ -96,34 +97,6 @@ void get_number_string(struct session *ses, char *str, char *result)
 	sprintf(result, "%.*f", precision, get_number(ses, str));
 }
 
-char *get_alnum(struct session *ses, char *str)
-{
-	char buffer[BUFFER_SIZE];
-	static char result[BUFFER_SIZE];
-	int sign;
-
-	sign = *str == '-' || *str == '+' ? TRUE : FALSE;
-
-	mathexp(ses, str, buffer);
-
-	if (is_number(buffer))
-	{
-		if (sign)
-		{
-			sprintf(result, "%+.*f", precision, tintoi(buffer));
-		}
-		else
-		{
-			sprintf(result, "%.*f", precision, tintoi(buffer));
-		}
-	}
-	else
-	{
-		strcpy(result, buffer);
-	}
-	return result;
-}
-
 double mathswitch(struct session *ses, char *left, char *right)
 {
 	char shift[BUFFER_SIZE];
@@ -171,7 +144,7 @@ void mathexp(struct session *ses, char *str, char *result)
 
 #define MATH_NODE(buffercheck, priority, newstatus)                  \
 {                                                                    \
-	if (buffercheck && (pta == buf3 || valid == FALSE))          \
+	if (buffercheck && pta == buf3)                              \
 	{                                                            \
 		return FALSE;                                        \
 	}                                                            \
@@ -181,7 +154,6 @@ void mathexp(struct session *ses, char *str, char *result)
 	add_math_node(buf1, buf2, buf3);                             \
 	status = newstatus;                                          \
 	pta = buf3;                                                  \
-	valid = FALSE;                                               \
 	point = -1;                                                  \
 }
 
@@ -212,12 +184,11 @@ void del_math_node(struct link_data *node)
 int mathexp_tokenize(struct session *ses, char *str)
 {
 	char buf1[BUFFER_SIZE], buf2[BUFFER_SIZE], buf3[STRING_SIZE], *pti, *pta;
-	int level, status, valid, point;
+	int level, status, point;
 
 	level     = 0;
 	point     = -1;
 	status    = EXP_VARIABLE;
-	valid     = FALSE;
 	precision = 0;
 
 	pta = buf3;
@@ -246,7 +217,6 @@ int mathexp_tokenize(struct session *ses, char *str)
 					case '8':
 					case '9':
 						*pta++ = *pti++;
-						valid = TRUE;
 
 						if (point >= 0)
 						{
@@ -276,7 +246,7 @@ int mathexp_tokenize(struct session *ses, char *str)
 					case '"':
 						if (pta != buf3)
 						{
-							show_message(ses, -1, "MATH EXP: \" FOUND INSIDE A NUMBER");
+//							show_message(ses, -1, "MATH EXP: \" FOUND INSIDE A NUMBER");
 							return FALSE;
 						}
 						*pta++ = *pti++;
@@ -286,7 +256,7 @@ int mathexp_tokenize(struct session *ses, char *str)
 					case '(':
 						if (pta != buf3)
 						{
-							show_message(ses, -1, "#MATH EXP: PARANTESES FOUND INSIDE A NUMBER");
+//							show_message(ses, -1, "#MATH EXP: PARANTESES FOUND INSIDE A NUMBER");
 							return FALSE;
 						}
 						*pta++ = *pti++;
@@ -298,11 +268,15 @@ int mathexp_tokenize(struct session *ses, char *str)
 						pti++;
 						break;
 
+					case ':':
+						*pta++ = *pti++;
+						break;
+						
 					case '.':
 						*pta++ = *pti++;
 						if (point >= 0)
 						{
-							show_message(ses, -1, "#MATH EXP: MORE THAN ONE POINT FOUND INSIDE A NUMBER");
+//							show_message(ses, -1, "#MATH EXP: MORE THAN ONE POINT FOUND INSIDE A NUMBER");
 							precision = 0;
 							return FALSE;
 						}
@@ -314,8 +288,29 @@ int mathexp_tokenize(struct session *ses, char *str)
 						pti++;
 						break;
 
+					case ')':
+					case 'd':
+					case '*':
+					case '/':
+					case '%':
+					case '<':
+					case '>':
+					case '&':
+					case '^':
+					case '|':
+					case '=':
+						if (pta != buf3)
+						{
+							MATH_NODE(FALSE, EXP_PR_VAR, EXP_OPERATOR);
+						}
+						else
+						{
+							*pta++ = *pti++;
+						}
+						break;
+
 					default:
-						MATH_NODE(TRUE, EXP_PR_VAR, EXP_OPERATOR);
+						*pta++ = *pti++;
 						break;
 				}
 				break;
@@ -325,7 +320,6 @@ int mathexp_tokenize(struct session *ses, char *str)
 				{
 					case '"':
 						*pta++ = *pti++;
-						valid = TRUE;
 						MATH_NODE(FALSE, EXP_PR_VAR, EXP_OPERATOR);
 						break;
 
@@ -483,7 +477,7 @@ int mathexp_tokenize(struct session *ses, char *str)
 
 	if (level != 0)
 	{
-		show_message(ses, -1, "#MATH EXP: UNMATCHED PARENTHESES, LEVEL: %d", level);
+//		show_message(ses, -1, "#MATH EXP: UNMATCHED PARENTHESES, LEVEL: %d", level);
 		return FALSE;
 	}
 
@@ -558,7 +552,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 		case 'd':
 			if (tintoi(node->next->str3) <= 0)
 			{
-				show_message(ses, -1, "#MATHEXP: INVALID DICE: %s", node->next->str3);
+//				show_message(ses, -1, "#MATHEXP: INVALID DICE: %s", node->next->str3);
 				value = 0;
 			}
 			else
@@ -572,7 +566,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 		case '/':
 			if (tintoi(node->next->str3) == 0)
 			{
-				show_message(ses, -1, "#MATH ERROR: DIVISION ZERO.");
+//				show_message(ses, -1, "#MATH ERROR: DIVISION ZERO.");
 				value = tintoi(node->prev->str3);
 			}
 			else
@@ -590,7 +584,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 		case '%':
 			if (tintoi(node->next->str3) == 0)
 			{
-				show_message(ses, -1, "#MATH ERROR: MODULO ZERO.");
+//				show_message(ses, -1, "#MATH ERROR: MODULO ZERO.");
 				value = tintoi(node->prev->str3);
 			}
 			else
@@ -662,6 +656,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 				case '|':
 					value = tintoi(node->prev->str3) || tintoi(node->next->str3);
 					break;
+
 				default:
 					value = (long long) tintoi(node->prev->str3) | (long long) tintoi(node->next->str3);
 					break;
@@ -674,7 +669,7 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 			value = (tineval(ses, node->prev->str3, node->next->str3) == 0);
 			break;
 		default:
-			show_message(ses, -1, "#COMPUTE EXP: UNKNOWN OPERATOR: %c%c", node->str3[0], node->str3[1]);
+//			show_message(ses, -1, "#COMPUTE EXP: UNKNOWN OPERATOR: %c%c", node->str3[0], node->str3[1]);
 			value = 0;
 			break;
 	}
@@ -701,9 +696,15 @@ void mathexp_compute(struct session *ses, struct link_data *node)
 	node->str3 = strdup(temp);
 }
 
+/*
+	Keep synced with is_number()
+*/
+
 double tintoi(char *str)
 {
 	char *ptr = str;
+	double values[5] = {0, 0, 0, 0, 0}, m = 1;
+	int i = 1, d = 0;
 
 	if (*ptr == 0)
 	{
@@ -720,6 +721,120 @@ double tintoi(char *str)
 			break;
 	}
 
+	ptr = str + strlen(str);
+
+	while (TRUE)
+	{
+		ptr--;
+
+		switch (*ptr)
+		{
+			case '0':
+			case '1':
+			case '2':
+			case '3':
+			case '4':
+			case '5':
+			case '6':
+			case '7':
+			case '8':
+			case '9':
+				values[i] += (*ptr - '0') * m;
+				m *= 10;
+				break;
+
+			case '.':
+				if (d)
+				{
+					return 0;
+				}
+				d = 1;
+
+				values[0] = values[1] / m;
+				values[1] = 0;
+				m = 1;
+				break;
+
+			case ':':
+				switch (i)
+				{
+					case 2:
+						values[i] *= 60;
+						break;
+					case 3:
+						values[i] *= 60 * 60;
+						break;
+					case 4:
+						return 0;
+				}
+				i++;
+				m = 1;
+				break;
+
+			case '!':
+			case '~':
+			case '+':
+			case '-':
+				if (ptr == str)
+				{
+					break;
+				}
+				return 0;
+
+			default:
+				return 0;
+		}
+
+		if (ptr == str)
+		{
+			switch (i)
+			{
+				case 2:
+					values[i] *= 60;
+					break;
+				case 3:
+					values[i] *= 60 * 60;
+					break;
+				case 4:
+					values[i] *= 60 * 60 * 24;
+					break;
+			}
+			break;
+		}
+	}
+
+	switch (*str)
+	{
+		case '!':
+			return !(values[0] + values[1] + values[2] + values[3] + values[4]);
+		case '~':
+			return ~ (long long) (values[0] + values[1] + values[2] + values[3] + values[4]);
+		case '+':
+			return +(values[0] + values[1] + values[2] + values[3] + values[4]);
+		case '-':
+			return -(values[0] + values[1] + values[2] + values[3] + values[4]);
+		default:
+			return (values[0] + values[1] + values[2] + values[3] + values[4]);
+	}
+
+	switch (str[0])
+	{
+		case '!':
+			return !atof(&str[1]);
+
+		case '~':
+			return (double) ~atoll(&str[1]);
+
+		case '+':
+			return +atof(&str[1]);
+
+		case '-':
+			return -atof(&str[1]);
+
+		default:
+			return atof(str);
+	}
+
 	while (*ptr)
 	{
 		if (!isdigit((int) *ptr))
@@ -781,7 +896,7 @@ double tineval(struct session *ses, char *left, char *right)
 	switch (left[0])
 	{
 		case '"':
-			return match(NULL, left, right);
+			return match(ses, left, right, SUB_NONE);
 
 		default:
 			return tintoi(left) == tintoi(right);
