@@ -35,7 +35,7 @@ DO_COMMAND(do_all)
 
 	if (gts->next)
 	{
-		get_arg_in_braces(arg, left, TRUE);
+		get_arg_in_braces(ses, arg, left, TRUE);
 
 		substitute(ses, left, left, SUB_VAR|SUB_FUN);
 
@@ -118,8 +118,8 @@ DO_COMMAND(do_echo)
 
 	if (*arg == DEFAULT_OPEN)
 	{
-		arg = get_arg_in_braces(arg, left, TRUE);
-		      get_arg_in_braces(arg, temp, TRUE);
+		arg = get_arg_in_braces(ses, arg, left, TRUE);
+		      get_arg_in_braces(ses, arg, temp, TRUE);
 
 		if (*temp)
 		{
@@ -137,7 +137,7 @@ DO_COMMAND(do_echo)
 
 	substitute(ses, arg, temp, SUB_ESC);
 
-	if (strip_vt102_strlen(ses->more_output) != 0)
+	if (strip_vt102_strlen(ses, ses->more_output) != 0)
 	{
 		sprintf(output, "\n\033[0m%s\033[0m", temp);
 	}
@@ -188,7 +188,7 @@ DO_COMMAND(do_forall)
 	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
 
 	arg = sub_arg_in_braces(ses, arg, left, GET_ALL, SUB_VAR|SUB_FUN);
-	arg = get_arg_in_braces(arg, right, TRUE);
+	arg = get_arg_in_braces(ses, arg, right, TRUE);
 
 	if (*left == 0 || *right == 0)
 	{
@@ -200,7 +200,7 @@ DO_COMMAND(do_forall)
 
 		while (*arg)
 		{
-			arg = get_arg_in_braces(arg, temp, TRUE);
+			arg = get_arg_in_braces(ses, arg, temp, TRUE);
 
 			RESTRING(gtd->cmds[0], temp);
 
@@ -260,10 +260,6 @@ DO_COMMAND(do_info)
 
 DO_COMMAND(do_nop)
 {
-	if (*arg == '-')
-	{
-		ses->input_level++;
-	}
 	return ses;
 }
 
@@ -271,10 +267,9 @@ DO_COMMAND(do_nop)
 DO_COMMAND(do_parse)
 {
 	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
-	int cnt;
 
-	arg = get_arg_in_braces(arg, left,  0);
-	arg = get_arg_in_braces(arg, right, 1);
+	arg = get_arg_in_braces(ses, arg, left,  0);
+	arg = get_arg_in_braces(ses, arg, right, 1);
 
 	substitute(ses, left, left, SUB_VAR|SUB_FUN);
 
@@ -284,22 +279,33 @@ DO_COMMAND(do_parse)
 	}
 	else
 	{
-		for (cnt = 0 ; left[cnt] != 0 ; cnt++)
-		{
-#ifdef BIG5
-			if (left[cnt] & 0x80 && left[cnt+1] != 0)
-			{
-				sprintf(temp, "%c%c", left[cnt], left[cnt+1]);
+		int i = 0;
 
-				cnt++;
+		while (left[i] != 0)
+		{
+			if (HAS_BIT(ses->flags, SES_FLAG_BIG5) && left[i] & 128 && left[i+1] != 0)
+			{
+				i += sprintf(temp, "%c%c", left[i], left[i+1]);
+			}
+			else if (HAS_BIT(ses->flags, SES_FLAG_UTF8) && (left[i] & 192) == 192 && left[i+1] != 0)
+			{
+				if ((left[i] & 240) == 240 && left[i+2] != 0 && left[i+3] != 0)
+				{
+					i += sprintf(temp, "%c%c%c%c", left[i], left[i+1], left[i+2], left[i+3]);
+				}
+				else if ((left[i] & 224) == 224 && left[i+2] != 0)
+				{
+					i += sprintf(temp, "%c%c%c", left[i], left[i+1], left[i+2]);
+				}
+				else
+				{
+					i += sprintf(temp, "%c%c", left[i], left[i+1]);
+				}
 			}
 			else
 			{
-				sprintf(temp, "%c", left[cnt]);
+				i += sprintf(temp, "%c", left[i]);
 			}
-#else
-			sprintf(temp, "%c", left[cnt]);
-#endif
 
 			RESTRING(gtd->cmds[0], temp);
 
@@ -318,7 +324,7 @@ DO_COMMAND(do_send)
 
 	push_call("do_send(%p,%p)",ses,arg);
 
-	get_arg_in_braces(arg, left, TRUE);
+	get_arg_in_braces(ses, arg, left, TRUE);
 
 	write_mud(ses, left, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
 
@@ -331,7 +337,7 @@ DO_COMMAND(do_showme)
 	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[STRING_SIZE];
 	int lnf;
 
-	arg = get_arg_in_braces(arg, left, TRUE);
+	arg = get_arg_in_braces(ses, arg, left, TRUE);
 
 	lnf = !str_suffix(left, "\\");
 
@@ -355,7 +361,7 @@ DO_COMMAND(do_showme)
 		return ses;
 	}
 
-	if (strip_vt102_strlen(ses->more_output) != 0)
+	if (strip_vt102_strlen(ses, ses->more_output) != 0)
 	{
 		sprintf(right, "\n\033[0m%s\033[0m", temp);
 	}
@@ -392,7 +398,7 @@ DO_COMMAND(do_snoop)
 	struct session *sesptr = ses;
 	char left[BUFFER_SIZE];
 
-	get_arg_in_braces(arg, left, 1);
+	get_arg_in_braces(ses, arg, left, 1);
 	substitute(ses, left, left, SUB_VAR|SUB_FUN);
 
 	if (*left)
@@ -439,14 +445,6 @@ DO_COMMAND(do_suspend)
 
 DO_COMMAND(do_test)
 {
-	char buf[BUFFER_SIZE];
-
-	strcpy(buf, get_alnum(ses, arg));
-
-	tintin_printf2(ses, "is_number: %d", is_number(buf));
-
-	tintin_printf2(ses, "%s", get_alnum(ses, buf));
-
 	return ses;
 }
 
