@@ -33,8 +33,8 @@ DO_COMMAND(do_prompt)
 	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE], arg3[BUFFER_SIZE];
 
 	arg = sub_arg_in_braces(ses, arg, arg1, 0, SUB_VAR|SUB_FUN);
-	arg = get_arg_in_braces(arg, arg2, 1);
-	arg = get_arg_in_braces(arg, arg3, 1);
+	arg = get_arg_in_braces(ses, arg, arg2, 1);
+	arg = get_arg_in_braces(ses, arg, arg3, 1);
 
 	if (*arg3 == 0)
 	{
