@@ -30,36 +30,33 @@
 
 DO_COMMAND(do_substitute)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], rank[BUFFER_SIZE], *str;
-	struct listroot *root;
+	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE], arg3[BUFFER_SIZE], *str;
 
-	root = ses->list[LIST_SUBSTITUTE];
+	str = sub_arg_in_braces(ses, arg, arg1, 0, SUB_VAR|SUB_FUN);
+	arg = get_arg_in_braces(ses, str, arg2, 1);
+	arg = get_arg_in_braces(ses, arg, arg3, 1);
 
-	str = get_arg_in_braces(arg, left,  0);
-	arg = get_arg_in_braces(str, right, 1);
-	arg = get_arg_in_braces(arg, rank,  1);
-
-	if (*rank == 0)
+	if (*arg3 == 0)
 	{
-		strcpy(rank, "5");
+		strcpy(arg3, "5");
 	}
 
-	if (*left == 0)
+	if (*arg1 == 0)
 	{
-		show_list(ses, root, LIST_SUBSTITUTE);
+		show_list(ses->list[LIST_SUBSTITUTE], 0);
 	}
 	else if (*str == 0)
 	{
-		if (show_node_with_wild(ses, left, LIST_SUBSTITUTE) == FALSE)
+		if (show_node_with_wild(ses, arg1, LIST_SUBSTITUTE) == FALSE)
 		{
-			show_message(ses, LIST_SUBSTITUTE, "#SUBSTITUTE: NO MATCH(ES) FOUND FOR {%s}.", left);
+			show_message(ses, LIST_SUBSTITUTE, "#SUBSTITUTE: NO MATCH(ES) FOUND FOR {%s}.", arg1);
 		}
 	}
 	else
 	{
-		updatenode_list(ses, left, right, rank, LIST_SUBSTITUTE);
+		update_node_list(ses->list[LIST_SUBSTITUTE], arg1, arg2, arg3);
 
-		show_message(ses, LIST_SUBSTITUTE, "#OK. {%s} IS NOW SUBSTITUTED AS {%s} @ {%s}.", left, right, rank);
+		show_message(ses, LIST_SUBSTITUTE, "#OK. {%s} IS NOW SUBSTITUTED AS {%s} @ {%s}.", arg1, arg2, arg3);
 	}
 	return ses;
 }
@@ -74,38 +71,58 @@ DO_COMMAND(do_unsubstitute)
 
 void check_all_substitutions(struct session *ses, char *original, char *line)
 {
+	char match[BUFFER_SIZE], subst[BUFFER_SIZE], output[BUFFER_SIZE], *ptl, *ptm, *pto;
+	struct listroot *root = ses->list[LIST_SUBSTITUTE];
 	struct listnode *node;
-	char match[BUFFER_SIZE], subst[BUFFER_SIZE], output[BUFFER_SIZE], *pt1, *pt2;
 
-	for (node = ses->list[LIST_SUBSTITUTE]->f_node ; node ; node = node->next)
+	for (root->update = 0 ; root->update < root->used ; root->update++)
 	{
-		if (check_one_regexp(ses, node, line, original, 0))
+		if (check_one_regexp(ses, root->list[root->update], line, original, 0))
 		{
-			if (*gtd->vars[0] == 0)
-			{
-				continue;
-			}
-			strcpy(match, gtd->vars[0]);
+			node = root->list[root->update];
+
+			pto = original;
+			ptl = line;
 
 			*output = 0;
 
-			substitute(ses, node->right, subst, SUB_ARG|SUB_VAR|SUB_FUN|SUB_COL|SUB_ESC);
-
-			pt1 = original;
-
-			while ((pt2 = strstr(pt1, match)) != NULL)
+			do
 			{
-				*pt2 = 0;
+				if (*gtd->vars[0] == 0)
+				{
+					break;
+				}
 
-				cat_sprintf(output, "%s%s", pt1, subst);
+				strcpy(match, gtd->vars[0]);
 
-				pt1 = pt2 + strlen(match);
+				substitute(ses, node->right, subst, SUB_ARG|SUB_VAR|SUB_FUN|SUB_COL|SUB_ESC);
+
+				ptm = strstr(pto, match);
+
+				if (!HAS_BIT(node->flags, NODE_FLAG_META))
+				{
+					if (ptm == NULL)
+					{
+						break;
+					}
+
+					ptl = strstr(ptl, match);
+					ptl = ptl + strlen(match);
+				}
+
+				*ptm = 0;
+
+				cat_sprintf(output, "%s%s", pto, subst);
+
+				pto = ptm + strlen(match);
+
+				show_debug(ses, LIST_SUBSTITUTE, "#DEBUG SUBSTITUTE {%s} {%s}", node->left, match);
 			}
-			strcat(output, pt1);
+			while (check_one_regexp(ses, node, ptl, pto, 0));
+
+			strcat(output, pto);
 
 			strcpy(original, output);
-
-			show_debug(ses, LIST_SUBSTITUTE, "#DEBUG SUBSTITUTE {%s} {%s}", node->left, match);
 		}
 	}
 }
